<!doctype html>
<html>

<head>
  <meta name="viewport" content="width=device-width" />
  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
  <title>$title</title>
  <style>
    body {
      background-color: #f6f6f6;
      font-family: sans-serif;
      font-size: 20px;
      line-height: 1.4;
      margin: 0;
      padding: 0;
    }

    img {
      max-width: 100%;
    }

    a {
      color: #ff5722;
      text-decoration: underline;
    }

    .container {
      width: 100%;
      vertical-align: top;
      margin-top: 9px;
    }

    .main {
      background: #ffffff;
      width: 100%;
      max-width: 600px;
      text-align: left;
      vertical-align: top;
      padding: 0px 8px;
    }

    .footer {
      padding-top: 9px;
      color: #999999;
      font-size: 14px;
      text-align: center;
    }

    .footer a {
      color: #999999;
    }

    .preheader {
      color: transparent;
      display: none;
      height: 0;
      max-height: 0;
      max-width: 0;
      opacity: 0;
      overflow: hidden;
      visibility: hidden;
      width: 0;
    }
  </style>
</head>

<body>
  <span class="preheader">$preheader</span>
  <table class="container">
    <tr>
      <td align="center">
        <table class="main">
          <tr>
            <td>$content</td>
          </tr>
        </table>
      </td>
      <tr>
        <td class="footer" align="center">
          Message envoyé par
          <a href="$INSTANCE_URL">$INSTANCE_NAME</a>
        </td>
      </tr>
  </table>
</body>

</html>