<!doctype html>
<html lang="fr">

<head>
  <meta http-equiv="X-UA-Compatible" content="IE=Edge" />
  <meta charset="utf-8">
  <title>$INSTANCE_NAME</title>
  <style type="text/css">
    html {
      background-color: #3f51b5;
      font-family: "Courier New", monospace;
    }

    h1 {
      margin-top: 0;
    }

    h1,
    a {
      color: #ff5722;
    }

    body {
      margin: 5% auto;
      padding: 1.5rem;
      max-width: 500px;
      background-color: #fefefe;
      border-radius: 5px;
    }
  </style>
</head>

<body>
  <h1>$INSTANCE_NAME</h1>
  <p>$INSTANCE_BASELINE</p>
  <dl>
    <dt>Pour s’abonner à un flux, envoyez&nbsp;:</dt>
    <dd>
      <em>$SUBSCRIBE_KEYWORD URL</em> à $BOT_EMAIL</dd>
    <dt>Pour se désabonner d’un flux, envoyez&nbsp;:</dt>
    <dd>
      <em>$UNSUBSCRIBE_KEYWORD URL</em> à $BOT_EMAIL</dd>
  </dl>
  <p>
    <a href="https://flumel.readthedocs.io" rel="external">Documentation</a>
    <a href="https://gitlab.com/canarduck/flumel" rel="external">Code source</a>
  </p>
</body>

</html>